/*****************************************************************************************************************************
--
--    File Name    : AXI_master_write_ch2.v

--    Description  : This module implements AXI master write controller for write channel 1 and 2
--                   

-- Targeted device                     : Microsemi-SoC
-- Author                              : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module AXI_master_write_ch #(parameter g_AXI_AWIDTH = 32,
                             parameter g_AXI_DWIDTH = 64,
                             parameter g_AXI_BUFF_AWIDTH = 12)
                           (input wire clock,
                            input wire nreset,
                            input wire[(g_AXI_AWIDTH - 1) : 0] axi_wr_ddr_addr,
                            input wire start_axi_tr,
                            input wire[4 : 0] burst_length,
                            input wire last_burst,
                            output reg transf_compl,
                            output reg master_wr_req,
                            input wire arbitr_ack,
                            output wire[(g_AXI_AWIDTH - 1) : 0] axi_daddr,
                            output wire[2 : 0] axi_tr_size,
                            output wire[1 : 0] axi_burst_typ,
                            output reg[3 : 0] axi_burst_len,
                            output wire[1 : 0] axi_tr_dir,
                            output reg axi_start,
                            input wire axi_tr_done,
                            input wire axi_data_ren,
                            output wire[(g_AXI_DWIDTH - 1) : 0] axi_wr_data_o,
                            output wire[(g_AXI_BUFF_AWIDTH - 1) : 0] axi_buff_raddr,
                            input wire[(g_AXI_DWIDTH - 1) : 0] axi_buff_datain);
  parameter[3 : 0] ii = 4'b1111;
  parameter[3 : 0] OOI = 4'b1110;
  parameter[3 : 0] IOI = 4'b1101;
  parameter[3 : 0] lOI = 4'b1100;
  parameter[3 : 0] oOI = 4'b1011;
  parameter[3 : 0] iOI = 4'b1010;
  parameter[3 : 0] OII = 4'b1001;
  parameter[3 : 0] III = 4'b1000;
  parameter[3 : 0] lII = 4'b0111;
  parameter[3 : 0] oII = 4'b0110;
  parameter[3 : 0] iII = 4'b0101;
  parameter[3 : 0] OlI = 4'b0100;
  parameter[3 : 0] IlI = 4'b0011;
  parameter[3 : 0] llI = 4'b0010;
  parameter[3 : 0] olI = 4'b0001;
  parameter[3 : 0] ilI = 4'b0000;
  parameter[2 : 0] O0I = 3'b011;
  parameter[1 : 0] loI = 2'b01;
  parameter[1 : 0] l0I = 2'b01;
  parameter[2 : 0] o0I = 3'd0,
                   ooI = 3'd1,
                   O1I = 3'd2,
                   I1I = 3'd3,
                   l1I = 3'd4,
                   o1I = 3'd5;
  reg[2 : 0] i1I;
  reg[(g_AXI_AWIDTH - 1) : 0] ioI;
  reg[4 : 0] IoI;
  reg[(g_AXI_BUFF_AWIDTH - 1) : 0] OiI;
  reg[(g_AXI_BUFF_AWIDTH - 1) : 0] IiI;
  reg liI;
  assign axi_burst_typ = l0I;
  assign axi_daddr = ioI;
  assign axi_tr_size = O0I;
  assign axi_tr_dir = loI;
  assign axi_wr_data_o = axi_buff_datain;
  assign axi_buff_raddr = axi_data_ren ? OiI : IiI;
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          IiI <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
        end
      else
        if (axi_data_ren)
          begin
            IiI <= OiI;
          end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          OiI <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
        end
      else
        begin
          if (last_burst && liI)
            OiI <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
          else
            if (axi_data_ren)
              OiI <= OiI + 1'b1;
            else
              OiI <= OiI;
        end
    end
  always
    @(posedge clock or
      negedge nreset)
    begin
      if (!nreset)
        begin
          i1I <= o0I;
        end
      else
        begin
          case (i1I)
            o0I:
              begin
                if (start_axi_tr)
                  begin
                    i1I <= ooI;
                  end
                else
                  begin
                    i1I <= o0I;
                  end
              end
            ooI:
              begin
                i1I <= O1I;
              end
            O1I:
              begin
                i1I <= I1I;
              end
            I1I:
              begin
                if (arbitr_ack)
                  begin
                    i1I <= l1I;
                  end
                else
                  begin
                    i1I <= I1I;
                  end
              end
            l1I:
              begin
                if (axi_tr_done)
                  begin
                    i1I <= o1I;
                  end
                else
                  begin
                    i1I <= l1I;
                  end
              end
            o1I:
              begin
                i1I <= o0I;
              end
            default:
              i1I <= o0I;
          endcase
        end
    end
  always
    @(posedge clock or
      negedge nreset)
    begin
      if (!nreset)
        begin
          ioI <= { g_AXI_AWIDTH { 1'b0 } };
          axi_start <= 1'b0;
          master_wr_req <= 1'b0;
          liI <= 1'b0;
          transf_compl <= 1'b0;
          IoI <= 5'd0;
          axi_burst_len <= ii;
        end
      else
        begin
          case (i1I)
            o0I:
              begin
                axi_start <= 1'b0;
                master_wr_req <= 1'b0;
                transf_compl <= 1'b0;
                liI <= 1'b0;
                axi_burst_len <= ii;
                if (start_axi_tr)
                  begin
                    ioI <= axi_wr_ddr_addr;
                    IoI <= burst_length;
                  end
                else
                  begin
                    ioI <= { g_AXI_AWIDTH { 1'b0 } };
                    IoI <= 5'd0;
                  end
              end
            ooI:
              begin
                case (IoI)
                  5'd1:
                    begin
                      axi_burst_len <= ilI;
                    end
                  5'd2:
                    begin
                      axi_burst_len <= olI;
                    end
                  5'd3:
                    begin
                      axi_burst_len <= llI;
                    end
                  5'd4:
                    begin
                      axi_burst_len <= IlI;
                    end
                  5'd5:
                    begin
                      axi_burst_len <= OlI;
                    end
                  5'd6:
                    begin
                      axi_burst_len <= iII;
                    end
                  5'd7:
                    begin
                      axi_burst_len <= oII;
                    end
                  5'd8:
                    begin
                      axi_burst_len <= lII;
                    end
                  5'd9:
                    begin
                      axi_burst_len <= III;
                    end
                  5'd10:
                    begin
                      axi_burst_len <= OII;
                    end
                  5'd11:
                    begin
                      axi_burst_len <= iOI;
                    end
                  5'd12:
                    begin
                      axi_burst_len <= oOI;
                    end
                  5'd13:
                    begin
                      axi_burst_len <= lOI;
                    end
                  5'd14:
                    begin
                      axi_burst_len <= IOI;
                    end
                  5'd15:
                    begin
                      axi_burst_len <= OOI;
                    end
                  5'd16:
                    begin
                      axi_burst_len <= ii;
                    end
                  default:
                    axi_burst_len <= ii;
                endcase
              end
            O1I:
              begin
                master_wr_req <= 1'b1;
                axi_start <= 1'b1;
              end
            I1I:
              begin
                if (arbitr_ack)
                  begin
                    axi_start <= 1'b0;
                  end
                else
                  begin
                    axi_start <= axi_start;
                  end
              end
            l1I:
              begin
                if (axi_tr_done)
                  begin
                    master_wr_req <= 1'b0;
                  end
                else
                  begin
                    master_wr_req <= master_wr_req;
                  end
              end
            o1I:
              begin
                transf_compl <= 1'b1;
                liI <= 1'b1;
              end
            default:
              begin
              end
          endcase
        end
    end
endmodule
