/*****************************************************************************************************************************
--
--    File Name    : ddr_displ_read_contrl.v

--    Description  : This module implements DDR read controller for Display read channel 1
--                   

-- Targeted device                     : Microsemi-SoC
-- Author                              : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module ddr_displ_read_contrl #(parameter g_AXI_AWIDTH = 32,
                               parameter g_AXI_BUFF_AWIDTH = 12)
                             (input wire clock,
                              input wire nreset,
                              input wire rd_req_i,
                              output reg rd_ack_o,
                              output reg rd_done_o,
                              input wire[(g_AXI_AWIDTH - 1) : 0] ddr_read_start_address_i,
                              input wire[(g_AXI_BUFF_AWIDTH + 3) - 1 : 0] bytes_to_read_i,
                              input wire unpacker_compl,
                              output reg start_unpack,
                              output wire[(g_AXI_BUFF_AWIDTH - 1) : 0] locs_to_read_to_unpacker,
                              output wire controller_ack,
                              output reg last_transact,
                              output wire[4 : 0] axi_burst_len,
                              input wire axi_tr_done,
                              output reg[(g_AXI_AWIDTH - 1) : 0] axi_rd_ddr_addr,
                              output reg start_axi_tr,
                              output wire[(g_AXI_BUFF_AWIDTH - 1) : 0] locs_to_read_to_axi,
                              output reg rd_burst_split);
  reg[(g_AXI_AWIDTH - 1) : 0] II0;
  reg[(g_AXI_BUFF_AWIDTH - 1) : 0] lI0;
  reg[(g_AXI_BUFF_AWIDTH - 1) : 0] oI0;
  reg[(g_AXI_BUFF_AWIDTH - 1) : 0] iI0;
  reg[5 : 0] Ol0;
  reg[4 : 0] Il0;
  reg[4 : 0] ll0;
  reg[(g_AXI_BUFF_AWIDTH - 1) : 0] ol0;
  reg[(g_AXI_BUFF_AWIDTH + 3) - 1 : 0] il0;
  reg O00;
  reg I00;
  reg l00;
  reg[(g_AXI_BUFF_AWIDTH - 1) : 0] o00;
  reg[(g_AXI_BUFF_AWIDTH - 1) : 0] i00;
  reg[(g_AXI_BUFF_AWIDTH - 1) : 0] O10;
  reg I10;
  reg l10;
  reg o10;
  reg[(g_AXI_AWIDTH - 1) : 0] i10;
  reg Oo0;
  reg Io0;
  reg lo0;
  parameter[31 : 0] oo0 = 32'd128;
  parameter[4 : 0] io0 = 5'd16;
  parameter[4 : 0] Oi0 = 5'd8;
  parameter[11 : 0] Ii0 = 12'hfff;
  parameter[4 : 0] li0 = 5'd0,
                   oi0 = 5'd1,
                   ii0 = 5'd2,
                   OO1 = 5'd3,
                   IO1 = 5'd4,
                   lO1 = 5'd5,
                   oO1 = 5'd6,
                   iO1 = 5'd7,
                   OI1 = 5'd8,
                   II1 = 5'd9,
                   lI1 = 5'd10,
                   oI1 = 5'd11,
                   iI1 = 5'd12,
                   Ol1 = 5'd13,
                   Il1 = 5'd14,
                   ll1 = 5'd15,
                   ol1 = 5'd16,
                   il1 = 5'd17,
                   O01 = 5'd18,
                   I01 = 5'd19,
                   l01 = 5'd20,
                   o01 = 5'd21;
  reg[4 : 0] i01;
  assign locs_to_read_to_axi = ol0;
  assign locs_to_read_to_unpacker = oI0;
  assign controller_ack = I00;
  assign axi_burst_len = Il0;
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          i01 <= li0;
        end
      else
        begin
          case (i01)
            li0:
              begin
                if (rd_req_i)
                  i01 <= oi0;
                else
                  i01 <= li0;
              end
            oi0:
              begin
                i01 <= ii0;
              end
            ii0:
              begin
                i01 <= OO1;
              end
            OO1:
              begin
                i01 <= IO1;
              end
            IO1:
              begin
                i01 <= lO1;
              end
            lO1:
              begin
                i01 <= oO1;
              end
            oO1:
              begin
                if (l10)
                  begin
                    if (I10)
                      begin
                        if (O00)
                          begin
                            i01 <= Il1;
                          end
                        else
                          begin
                            i01 <= iI1;
                          end
                      end
                    else
                      i01 <= II1;
                  end
                else
                  begin
                    i01 <= iO1;
                  end
              end
            iO1:
              begin
                i01 <= OI1;
              end
            OI1:
              begin
                if (I10)
                  begin
                    if (O00)
                      begin
                        i01 <= I01;
                      end
                    else
                      begin
                        i01 <= iI1;
                      end
                  end
                else
                  i01 <= II1;
              end
            II1:
              begin
                if (O00)
                  i01 <= o01;
                else
                  begin
                    if (axi_tr_done)
                      begin
                        i01 <= o01;
                      end
                    else
                      i01 <= II1;
                  end
              end
            o01:
              begin
                i01 <= lI1;
              end
            lI1:
              begin
                if (o10)
                  begin
                    if (Ol0 == 6'd48)
                      begin
                        i01 <= oI1;
                      end
                    else
                      begin
                        i01 <= OI1;
                      end
                  end
                else
                  begin
                    if (Ol0 == 6'd48)
                      begin
                        i01 <= oI1;
                      end
                    else
                      begin
                        i01 <= OO1;
                      end
                  end
              end
            oI1:
              begin
                i01 <= l01;
              end
            l01:
              begin
                if (o10)
                  begin
                    i01 <= OI1;
                  end
                else
                  begin
                    i01 <= OO1;
                  end
              end
            iI1:
              begin
                if (axi_tr_done)
                  begin
                    if (o10)
                      begin
                        if (Oo0)
                          i01 <= Il1;
                        else
                          i01 <= Ol1;
                      end
                    else
                      begin
                        i01 <= Il1;
                      end
                  end
                else
                  i01 <= iI1;
              end
            Ol1:
              begin
                i01 <= OI1;
              end
            I01:
              begin
                i01 <= Ol1;
              end
            Il1:
              begin
                if (axi_tr_done)
                  i01 <= ll1;
                else
                  i01 <= Il1;
              end
            ll1:
              begin
                if (l00)
                  begin
                    if (unpacker_compl)
                      begin
                        i01 <= ol1;
                      end
                    else
                      begin
                        i01 <= ll1;
                      end
                  end
                else
                  begin
                    i01 <= ol1;
                  end
              end
            ol1:
              begin
                i01 <= il1;
              end
            il1:
              begin
                if (unpacker_compl)
                  i01 <= O01;
                else
                  i01 <= il1;
              end
            O01:
              begin
                i01 <= li0;
              end
            default:
              i01 <= li0;
          endcase
        end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          axi_rd_ddr_addr <= { g_AXI_AWIDTH { 1'b0 } };
          start_axi_tr <= 1'b0;
          start_unpack <= 1'b0;
          lI0 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
          oI0 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
          Il0 <= io0;
          ol0 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
          II0 <= { g_AXI_AWIDTH { 1'b0 } };
          i10 <= { g_AXI_AWIDTH { 1'b0 } };
          rd_ack_o <= 1'b0;
          rd_done_o <= 1'b0;
          il0 <= { (g_AXI_BUFF_AWIDTH + 3) { 1'b0 } };
          O00 <= 1'b0;
          I00 <= 1'b0;
          last_transact <= 1'b0;
          l00 <= 1'b0;
          rd_burst_split <= 1'b0;
          o00 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
          ll0 <= 5'd0;
          I10 <= 1'b0;
          i00 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
          l10 <= 1'b0;
          O10 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
          o10 <= 1'b0;
          Oo0 <= 1'b0;
          Ol0 <= 6'd0;
          iI0 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
          Io0 <= 1'b0;
          lo0 <= 1'b0;
        end
      else
        begin
          case (i01)
            li0:
              begin
                axi_rd_ddr_addr <= { g_AXI_AWIDTH { 1'b0 } };
                start_axi_tr <= 1'b0;
                start_unpack <= 1'b0;
                lI0 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
                oI0 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
                Il0 <= io0;
                ol0 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
                II0 <= { g_AXI_AWIDTH { 1'b0 } };
                i10 <= { g_AXI_AWIDTH { 1'b0 } };
                rd_ack_o <= 1'b0;
                rd_done_o <= 1'b0;
                O00 <= 1'b1;
                il0 <= { (g_AXI_BUFF_AWIDTH + 3) { 1'b0 } };
                I00 <= 1'b0;
                last_transact <= 1'b0;
                l00 <= 1'b0;
                I10 <= 1'b0;
                rd_burst_split <= 1'b0;
                o00 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
                ll0 <= 5'd0;
                i00 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
                l10 <= 1'b0;
                O10 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
                o10 <= 1'b0;
                Oo0 <= 1'b0;
                Ol0 <= 6'd0;
                iI0 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
                Io0 <= 1'b0;
                lo0 <= 1'b0;
              end
            oi0:
              begin
                II0 <= ddr_read_start_address_i;
                il0 <= bytes_to_read_i;
                rd_ack_o <= 1'b1;
              end
            ii0:
              begin
                rd_ack_o <= 1'b0;
                lI0 <= il0 >> 3;
                ol0 <= il0 >> 3;
              end
            OO1:
              begin
                start_unpack <= 1'b0;
                if (lI0 > io0)
                  begin
                    ll0 <= io0;
                    I10 <= 1'b0;
                  end
                else
                  begin
                    ll0 <= lI0[4 : 0];
                    I10 <= 1'b1;
                  end
              end
            IO1:
              begin
                o00 <= Ii0 - II0[11 : 0];
                i00 <= ll0 * Oi0;
              end
            lO1:
              begin
                if (o00 >= (i00 - 1'b1))
                  l10 <= 1'b1;
                else
                  l10 <= 1'b0;
              end
            oO1:
              begin
                if (l10)
                  begin
                    Il0 <= ll0;
                    if (I10 && O00)
                      begin
                        start_axi_tr <= 1'b1;
                        O00 <= 1'b0;
                        axi_rd_ddr_addr <= II0;
                      end
                    else
                      begin
                        start_axi_tr <= start_axi_tr;
                        O00 <= O00;
                        axi_rd_ddr_addr <= axi_rd_ddr_addr;
                      end
                  end
                else
                  begin
                    o00 <= o00 + 12'd1;
                  end
              end
            iO1:
              begin
                i10 <= II0;
                O10 <= o00 >> 3;
              end
            OI1:
              begin
                Il0 <= O10[4 : 0];
                o10 <= 1'b1;
                if (I10 && O00)
                  begin
                    axi_rd_ddr_addr <= i10;
                    start_axi_tr <= 1'b1;
                  end
                else
                  begin
                    axi_rd_ddr_addr <= axi_rd_ddr_addr;
                    start_axi_tr <= 1'b0;
                  end
              end
            II1:
              begin
                if (O00)
                  begin
                    axi_rd_ddr_addr <= II0;
                    start_axi_tr <= 1'b1;
                    O00 <= O00;
                  end
                else
                  begin
                    if (axi_tr_done)
                      begin
                        if (o10)
                          begin
                            axi_rd_ddr_addr <= i10;
                            start_axi_tr <= 1'b1;
                            O00 <= 1'b0;
                            if (Oo0)
                              begin
                                o10 <= 1'b0;
                                rd_burst_split <= 1'b0;
                                lo0 <= 1'b0;
                                Io0 <= 1'b1;
                              end
                            else
                              begin
                                o10 <= o10;
                                rd_burst_split <= 1'b1;
                                Io0 <= 1'b0;
                                lo0 <= 1'b1;
                              end
                          end
                        else
                          begin
                            axi_rd_ddr_addr <= II0;
                            start_axi_tr <= 1'b1;
                            O00 <= 1'b0;
                            Io0 <= Io0;
                            if (Io0)
                              lo0 <= 1'b0;
                            else
                              lo0 <= 1'b1;
                          end
                      end
                    else
                      begin
                        axi_rd_ddr_addr <= axi_rd_ddr_addr;
                        O00 <= 1'b0;
                        start_axi_tr <= 1'b0;
                      end
                  end
              end
            o01:
              begin
                if (O00)
                  begin
                    O00 <= 1'b0;
                    Ol0 <= Ol0;
                  end
                else
                  if (Io0 & !Oo0)
                    begin
                      Ol0 <= Ol0 + 6'd16;
                      Io0 <= 1'b0;
                    end
                  else
                    if (lo0)
                      begin
                        Ol0 <= Ol0 + 6'd16;
                      end
                    else
                      begin
                        Ol0 <= Ol0;
                      end
              end
            lI1:
              begin
                start_axi_tr <= 1'b0;
                if (o10)
                  begin
                    i10 <= i10 + o00;
                    O10 <= ll0 - O10;
                    Oo0 <= 1'b1;
                  end
                else
                  begin
                    lI0 <= lI0 - io0;
                    II0 <= II0 + oo0;
                    Oo0 <= 1'b0;
                  end
              end
            oI1:
              begin
                start_axi_tr <= 1'b0;
                if (l00)
                  begin
                    if (unpacker_compl)
                      begin
                        I00 <= 1'b1;
                        l00 <= 1'b1;
                        iI0 <= iI0 + Ol0;
                        oI0 <= Ol0;
                        start_unpack <= 1'b1;
                      end
                    else
                      begin
                        I00 <= 1'b0;
                        l00 <= 1'b1;
                        iI0 <= iI0;
                        oI0 <= oI0;
                        start_unpack <= 1'b0;
                      end
                  end
                else
                  begin
                    l00 <= 1'b1;
                    iI0 <= iI0 + Ol0;
                    oI0 <= Ol0;
                    start_unpack <= 1'b1;
                    I00 <= 1'b0;
                  end
              end
            l01:
              begin
                I00 <= 1'b0;
                Ol0 <= 6'd0;
              end
            iI1:
              begin
                if (axi_tr_done)
                  begin
                    axi_rd_ddr_addr <= II0;
                    start_axi_tr <= 1'b1;
                  end
                else
                  begin
                    axi_rd_ddr_addr <= axi_rd_ddr_addr;
                    start_axi_tr <= 1'b0;
                  end
              end
            Ol1:
              begin
                start_axi_tr <= 1'b0;
                Oo0 <= 1'b1;
                rd_burst_split <= 1'b1;
                II0 <= II0 + o00;
                O10 <= ll0 - O10;
              end
            I01:
              begin
                O00 <= 1'b0;
                start_axi_tr <= 1'b0;
              end
            Il1:
              begin
                start_axi_tr <= 1'b0;
                rd_burst_split <= 1'b0;
                Oo0 <= 1'b0;
                o10 <= 1'b0;
              end
            ll1:
              begin
                if (l00)
                  begin
                    if (unpacker_compl)
                      begin
                        I00 <= 1'b1;
                        oI0 <= ol0 - iI0;
                      end
                    else
                      begin
                        I00 <= 1'b0;
                        oI0 <= oI0;
                      end
                  end
                else
                  begin
                    oI0 <= ol0;
                    I00 <= 1'b0;
                  end
              end
            ol1:
              begin
                I00 <= 1'b0;
                start_unpack <= 1'b1;
              end
            il1:
              begin
                start_unpack <= 1'b0;
              end
            O01:
              begin
                I00 <= 1'b1;
                last_transact <= 1'b1;
                rd_done_o <= 1'b1;
              end
            default:
              begin
              end
          endcase
        end
    end
endmodule
