/*****************************************************************************************************************************
--
--    File Name    : ddr_memory_arbiter.v  

--    Description  : This module is the ddr arbiter top level
--                   

-- Targeted device                     : Microsemi-SoC
-- Author                              : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module ddr_memory_arbiter #(parameter g_AXI_AWIDTH = 32,
                            parameter g_AXI_DWIDTH = 64,
                            parameter g_RD_CHANNEL1_AXI_BUFF_AWIDTH = 13,
                            parameter g_RD_CHANNEL2_AXI_BUFF_AWIDTH = 13,
                            parameter g_RD_CHANNEL3_AXI_BUFF_AWIDTH = 13,
                            parameter g_RD_CHANNEL4_AXI_BUFF_AWIDTH = 13,
                            parameter g_WR_CHANNEL1_AXI_BUFF_AWIDTH = 13,
                            parameter g_WR_CHANNEL2_AXI_BUFF_AWIDTH = 13,
                            parameter g_RD_CHANNEL1_HORIZONTAL_RESOLUTION = 1280,
                            parameter g_RD_CHANNEL2_HORIZONTAL_RESOLUTION = 1280,
                            parameter g_RD_CHANNEL3_HORIZONTAL_RESOLUTION = 1280,
                            parameter g_RD_CHANNEL4_HORIZONTAL_RESOLUTION = 1280,
                            parameter g_WR_CHANNEL1_HORIZONTAL_RESOLUTION = 1280,
                            parameter g_WR_CHANNEL2_HORIZONTAL_RESOLUTION = 1280,
                            parameter g_RD_CHANNEL1_VIDEO_DATA_WIDTH = 24,
                            parameter g_RD_CHANNEL2_VIDEO_DATA_WIDTH = 24,
                            parameter g_RD_CHANNEL3_VIDEO_DATA_WIDTH = 32,
                            parameter g_RD_CHANNEL4_VIDEO_DATA_WIDTH = 8,
                            parameter g_WR_CHANNEL1_VIDEO_DATA_WIDTH = 32,
                            parameter g_WR_CHANNEL2_VIDEO_DATA_WIDTH = 32,
                            parameter g_RD_CHANNEL1_BUFFER_LINE_STORAGE = 2,
                            parameter g_RD_CHANNEL2_BUFFER_LINE_STORAGE = 1,
                            parameter g_RD_CHANNEL3_BUFFER_LINE_STORAGE = 1,
                            parameter g_RD_CHANNEL4_BUFFER_LINE_STORAGE = 2,
                            parameter g_WR_CHANNEL1_BUFFER_LINE_STORAGE = 1,
                            parameter g_WR_CHANNEL2_BUFFER_LINE_STORAGE = 1)
                          (input wire RESET_N_I,
                           input wire SYS_CLOCK_I,
                           input wire BUFF_READ_CLOCK_I,
                           input wire rd_req_1_i,
                           input wire[(g_AXI_AWIDTH - 1) : 0] start_read_addr_1_i,
                           input wire[(g_RD_CHANNEL1_AXI_BUFF_AWIDTH + 3) - 1 : 0] bytes_to_read_1_i,
                           input wire rd_req_2_i,
                           input wire[(g_AXI_AWIDTH - 1) : 0] start_read_addr_2_i,
                           input wire[(g_RD_CHANNEL2_AXI_BUFF_AWIDTH + 3) - 1 : 0] bytes_to_read_2_i,
                           input wire rd_req_3_i,
                           input wire[(g_AXI_AWIDTH - 1) : 0] start_read_addr_3_i,
                           input wire[(g_RD_CHANNEL3_AXI_BUFF_AWIDTH + 3) - 1 : 0] bytes_to_read_3_i,
                           input wire rd_req_4_i,
                           input wire[(g_AXI_AWIDTH - 1) : 0] start_read_addr_4_i,
                           input wire[(g_RD_CHANNEL4_AXI_BUFF_AWIDTH + 3) - 1 : 0] bytes_to_read_4_i,
                           input wire wr_req_1_i,
                           input wire[(g_AXI_AWIDTH - 1) : 0] start_write_addr_1_i,
                           input wire[(g_WR_CHANNEL1_AXI_BUFF_AWIDTH + 3) - 1 : 0] bytes_to_write_1_i,
                           input wire[(g_WR_CHANNEL1_VIDEO_DATA_WIDTH - 1) : 0] video_wdata_1_i,
                           input wire wdata_valid_1_i,
                           input wire wr_req_2_i,
                           input wire[(g_AXI_AWIDTH - 1) : 0] start_write_addr_2_i,
                           input wire[(g_WR_CHANNEL2_AXI_BUFF_AWIDTH + 3) - 1 : 0] bytes_to_write_2_i,
                           input wire[(g_WR_CHANNEL2_VIDEO_DATA_WIDTH - 1) : 0] video_wdata_2_i,
                           input wire wdata_valid_2_i,
                           input wire m_arready_i,
                           input wire m_awready_i,
                           input wire[3 : 0] m_bid_i,
                           input wire[1 : 0] m_bresp_i,
                           input wire m_bvalid_i,
                           input wire[3 : 0] m_rid_i,
                           input wire[(g_AXI_DWIDTH - 1) : 0] m_rdata_i,
                           input wire m_rlast_i,
                           input wire[1 : 0] m_rresp_i,
                           input wire m_rvalid_i,
                           input wire m_wready_i,
                           output wire rd_ack_1_o,
                           output wire rd_done_1_o,
                           output wire[(g_RD_CHANNEL1_VIDEO_DATA_WIDTH - 1) : 0] video_rdata_1_o,
                           output wire rdata_valid_1_o,
                           output wire rd_ack_2_o,
                           output wire rd_done_2_o,
                           output wire[(g_RD_CHANNEL2_VIDEO_DATA_WIDTH - 1) : 0] video_rdata_2_o,
                           output wire rdata_valid_2_o,
                           output wire rd_ack_3_o,
                           output wire rd_done_3_o,
                           output wire[(g_RD_CHANNEL3_VIDEO_DATA_WIDTH - 1) : 0] video_rdata_3_o,
                           output wire rdata_valid_3_o,
                           output wire rd_ack_4_o,
                           output wire rd_done_4_o,
                           output wire[(g_RD_CHANNEL4_VIDEO_DATA_WIDTH - 1) : 0] video_rdata_4_o,
                           output wire rdata_valid_4_o,
                           output wire wr_ack_1_o,
                           output wire wr_done_1_o,
                           output wire wr_ack_2_o,
                           output wire wr_done_2_o,
                           output wire[3 : 0] m_arid_o,
                           output wire[(g_AXI_AWIDTH - 1) : 0] m_araddr_o,
                           output wire[1 : 0] m_arburst_o,
                           output wire[3 : 0] m_arcache_o,
                           output wire[3 : 0] m_arlen_o,
                           output wire[1 : 0] m_arlock_o,
                           output wire[2 : 0] m_arprot_o,
                           output wire[2 : 0] m_arsize_o,
                           output wire m_arvalid_o,
                           output wire m_rready_o,
                           output wire[3 : 0] m_awid_o,
                           output wire[31 : 0] m_awaddr_o,
                           output wire[1 : 0] m_awburst_o,
                           output wire[3 : 0] m_awcache_o,
                           output wire[3 : 0] m_awlen_o,
                           output wire[1 : 0] m_awlock_o,
                           output wire[2 : 0] m_awprot_o,
                           output wire[2 : 0] m_awsize_o,
                           output wire m_awvalid_o,
                           output wire[3 : 0] m_wid_o,
                           output wire[(g_AXI_DWIDTH - 1) : 0] m_wdata_o,
                           output wire m_wlast_o,
                           output wire[7 : 0] m_wstrb_o,
                           output wire m_wvalid_o,
                           output wire m_bready_o);
  wire O11;
  wire I11;
  wire l11;
  wire o11;
  wire i11;
  wire Oo1;
  wire[(g_AXI_DWIDTH - 1) : 0] Io1;
  wire[(g_AXI_DWIDTH - 1) : 0] lo1;
  wire[(g_AXI_DWIDTH - 1) : 0] oo1;
  wire[(g_AXI_DWIDTH - 1) : 0] io1;
  wire Oi1;
  wire Ii1;
  wire li1;
  wire oi1;
  wire[1 : 0] ii1;
  wire[1 : 0] OOo;
  wire[(g_AXI_AWIDTH - 1) : 0] IOo;
  wire lOo;
  wire oOo;
  wire iOo;
  wire OIo;
  wire IIo;
  wire lIo;
  wire[3 : 0] oIo;
  wire[3 : 0] iIo;
  wire[(g_AXI_AWIDTH - 1) : 0] Olo;
  wire[2 : 0] Ilo;
  wire[2 : 0] llo;
  wire olo;
  wire ilo;
  wire[1 : 0] O0o;
  wire[1 : 0] I0o;
  wire[(g_AXI_DWIDTH - 1) : 0] l0o;
  wire o0o;
  wire i0o;
  wire O1o;
  wire I1o;
  wire[(g_AXI_DWIDTH - 1) : 0] l1o;
  wire o1o;
  wire i1o;
  wire[(g_AXI_AWIDTH - 1) : 0] Ooo;
  wire[1 : 0] Ioo;
  wire[3 : 0] loo;
  wire[3 : 0] ooo;
  wire[3 : 0] ioo;
  wire[1 : 0] Oio;
  wire[2 : 0] Iio;
  wire[2 : 0] lio;
  wire oio;
  wire[(g_AXI_AWIDTH - 1) : 0] iio;
  wire[1 : 0] OOi;
  wire[3 : 0] IOi;
  wire[3 : 0] lOi;
  wire[3 : 0] oOi;
  wire[1 : 0] iOi;
  wire[2 : 0] OIi;
  wire[2 : 0] IIi;
  wire lIi;
  wire oIi;
  wire iIi;
  wire[(g_AXI_DWIDTH - 1) : 0] Oli;
  wire[3 : 0] Ili;
  wire lli;
  wire[7 : 0] oli;
  wire ili;
  wire O0i;
  wire I0i;
  wire l0i;
  wire o0i;
  wire i0i;
  wire O1i;
  wire I1i;
  wire l1i;
  wire o1i;
  wire i1i;
  wire Ooi;
  wire Ioi;
  wire loi;
  wire ooi;
  wire ioi;
  wire Oii;
  wire[3 : 0] Iii;
  wire[1 : 0] lii;
  wire[(g_AXI_AWIDTH - 1) : 0] oii;
  wire iii;
  wire[1 : 0] OOOI;
  wire[2 : 0] IOOI;
  wire[3 : 0] lOOI;
  wire[1 : 0] oOOI;
  wire[(g_AXI_AWIDTH - 1) : 0] iOOI;
  wire OIOI;
  wire[1 : 0] IIOI;
  wire[2 : 0] lIOI;
  wire oIOI;
  wire[3 : 0] iIOI;
  wire[1 : 0] OlOI;
  wire[(g_AXI_AWIDTH - 1) : 0] IlOI;
  wire llOI;
  wire[1 : 0] olOI;
  wire[2 : 0] ilOI;
  wire O0OI;
  wire[3 : 0] I0OI;
  wire[1 : 0] l0OI;
  wire[(g_AXI_AWIDTH - 1) : 0] o0OI;
  wire i0OI;
  wire[1 : 0] O1OI;
  wire[2 : 0] I1OI;
  wire l1OI;
  wire o1OI;
  wire i1OI;
  wire OoOI;
  wire IoOI;
  wire[3 : 0] loOI;
  wire[1 : 0] ooOI;
  wire[(g_AXI_AWIDTH - 1) : 0] ioOI;
  wire OiOI;
  wire[1 : 0] IiOI;
  wire[2 : 0] liOI;
  wire[(g_AXI_DWIDTH - 1) : 0] oiOI;
  wire iiOI;
  wire[3 : 0] OOII;
  wire[1 : 0] IOII;
  wire[(g_AXI_AWIDTH - 1) : 0] lOII;
  wire oOII;
  wire[1 : 0] iOII;
  wire[2 : 0] OIII;
  wire[(g_AXI_DWIDTH - 1) : 0] IIII;
  wire lIII;
  wire oIII;
  wire iIII;
  wire OlII;
  wire IlII;
  wire llII;
  wire olII;
  wire ilII;
  wire O0II;
  wire I0II;
  wire l0II;
  wire o0II;
  wire i0II;
  wire O1II;
  wire I1II;
  wire l1II;
  wire o1II;
  wire i1II;
  wire OoII;
  wire IoII;
  wire loII;
  wire ooII;
  wire ioII;
  wire[3 : 0] OiII;
  wire[(g_AXI_AWIDTH - 1) : 0] IiII;
  wire[3 : 0] liII;
  wire[2 : 0] oiII;
  wire[1 : 0] iiII;
  wire[1 : 0] OOlI;
  wire[3 : 0] IOlI;
  wire[2 : 0] lOlI;
  wire[3 : 0] oOlI;
  wire[(g_AXI_DWIDTH - 1) : 0] iOlI;
  wire[7 : 0] OIlI;
  wire[3 : 0] IIlI;
  wire[(g_AXI_AWIDTH - 1) : 0] lIlI;
  wire[3 : 0] oIlI;
  wire[2 : 0] iIlI;
  wire[1 : 0] OllI;
  wire[1 : 0] IllI;
  wire[3 : 0] lllI;
  wire[2 : 0] ollI;
  wire[(g_RD_CHANNEL1_VIDEO_DATA_WIDTH - 1) : 0] illI;
  wire[(g_RD_CHANNEL2_VIDEO_DATA_WIDTH - 1) : 0] O0lI;
  wire[(g_RD_CHANNEL3_VIDEO_DATA_WIDTH - 1) : 0] I0lI;
  wire[(g_RD_CHANNEL4_VIDEO_DATA_WIDTH - 1) : 0] l0lI;
  wire[(g_RD_CHANNEL1_VIDEO_DATA_WIDTH - 1) : 0] o0lI;
  wire[(g_RD_CHANNEL2_VIDEO_DATA_WIDTH - 1) : 0] i0lI;
  wire[(g_RD_CHANNEL3_VIDEO_DATA_WIDTH - 1) : 0] O1lI;
  wire[(g_RD_CHANNEL4_VIDEO_DATA_WIDTH - 1) : 0] I1lI;
  wire l1lI;
  wire o1lI;
  assign oIII = lIi;
  assign m_awvalid_o = oIII;
  assign iIII = lli;
  assign m_wlast_o = iIII;
  assign OlII = ili;
  assign m_wvalid_o = OlII;
  assign IlII = oIi;
  assign m_bready_o = IlII;
  assign llII = oio;
  assign m_arvalid_o = llII;
  assign olII = iIi;
  assign m_rready_o = olII;
  assign I0II = O0i;
  assign rd_ack_1_o = I0II;
  assign ilII = loi;
  assign rdata_valid_1_o = ilII;
  assign O0II = i0i;
  assign rd_done_1_o = O0II;
  assign l0II = I0i;
  assign rd_ack_2_o = l0II;
  assign o0II = ooi;
  assign rdata_valid_2_o = o0II;
  assign i0II = O1i;
  assign rd_done_2_o = i0II;
  assign i1II = l0i;
  assign rd_ack_3_o = i1II;
  assign O1II = ioi;
  assign rdata_valid_3_o = O1II;
  assign l1II = I1i;
  assign rd_done_3_o = l1II;
  assign OoII = o0i;
  assign rd_ack_4_o = OoII;
  assign I1II = Oii;
  assign rdata_valid_4_o = I1II;
  assign o1II = l1i;
  assign rd_done_4_o = o1II;
  assign loII = o1OI;
  assign wr_ack_1_o = loII;
  assign IoII = OoOI;
  assign wr_done_1_o = IoII;
  assign ioII = i1OI;
  assign wr_ack_2_o = ioII;
  assign ooII = IoOI;
  assign wr_done_2_o = ooII;
  assign OiII = lOi;
  assign m_awid_o[3 : 0] = OiII;
  assign IiII = iio;
  assign m_awaddr_o[31 : 0] = IiII;
  assign liII = oOi;
  assign m_awlen_o[3 : 0] = liII;
  assign oiII = IIi;
  assign m_awsize_o[2 : 0] = oiII;
  assign iiII = OOi;
  assign m_awburst_o[1 : 0] = iiII;
  assign OOlI = iOi;
  assign m_awlock_o[1 : 0] = OOlI;
  assign IOlI = IOi;
  assign m_awcache_o[3 : 0] = IOlI;
  assign lOlI = OIi;
  assign m_awprot_o[2 : 0] = lOlI;
  assign oOlI = Ili;
  assign m_wid_o[3 : 0] = oOlI;
  assign iOlI = Oli;
  assign m_wdata_o[63 : 0] = iOlI;
  assign OIlI = oli;
  assign m_wstrb_o[7 : 0] = OIlI;
  assign IIlI = ooo;
  assign m_arid_o[3 : 0] = IIlI;
  assign lIlI = Ooo;
  assign m_araddr_o[31 : 0] = lIlI;
  assign oIlI = ioo;
  assign m_arlen_o[3 : 0] = oIlI;
  assign iIlI = lio;
  assign m_arsize_o[2 : 0] = iIlI;
  assign OllI = Ioo;
  assign m_arburst_o[1 : 0] = OllI;
  assign IllI = Oio;
  assign m_arlock_o[1 : 0] = IllI;
  assign lllI = loo;
  assign m_arcache_o[3 : 0] = lllI;
  assign ollI = Iio;
  assign m_arprot_o[2 : 0] = ollI;
  assign o0lI = illI;
  assign video_rdata_1_o = o0lI;
  assign i0lI = O0lI;
  assign video_rdata_2_o = i0lI;
  assign O1lI = I0lI;
  assign video_rdata_3_o = O1lI;
  assign I1lI = l0lI;
  assign video_rdata_4_o = I1lI;
  axi_arbiter #(.g_AXI_AWIDTH(g_AXI_AWIDTH), .g_AXI_DWIDTH(g_AXI_DWIDTH)) i1lI(.clock(SYS_CLOCK_I),
                                                                               .nreset(RESET_N_I),
                                                                               .master1_wr_req(l1lI),
                                                                               .master2_wr_req(o1lI),
                                                                               .master1_rd_req(o1i),
                                                                               .master2_rd_req(i1i),
                                                                               .master3_rd_req(Ooi),
                                                                               .master4_rd_req(Ioi),
                                                                               .split_burst_wr1(iiOI),
                                                                               .split_burst_wr2(lIII),
                                                                               .split_burst_rd2(oIOI),
                                                                               .split_burst_rd3(O0OI),
                                                                               .split_burst_rd4(l1OI),
                                                                               .tr_trg_w1(OiOI),
                                                                               .tr_trg_w2(oOII),
                                                                               .tr_trg_r1(iii),
                                                                               .tr_trg_r2(OIOI),
                                                                               .tr_trg_r3(llOI),
                                                                               .tr_trg_r4(i0OI),
                                                                               .tr_done_w(i1o),
                                                                               .tr_done_r(o1o),
                                                                               .buff_re(O1o),
                                                                               .buff_we(I1o),
                                                                               .tr_length_w1(loOI),
                                                                               .tr_length_w2(OOII),
                                                                               .tr_length_r1(Iii),
                                                                               .tr_length_r2(lOOI),
                                                                               .tr_length_r3(iIOI),
                                                                               .tr_length_r4(I0OI),
                                                                               .tr_size_w1(liOI),
                                                                               .tr_size_w2(OIII),
                                                                               .tr_size_r1(IOOI),
                                                                               .tr_size_r2(lIOI),
                                                                               .tr_size_r3(ilOI),
                                                                               .tr_size_r4(I1OI),
                                                                               .tr_burst_w1(ooOI),
                                                                               .tr_burst_w2(IOII),
                                                                               .tr_burst_r1(lii),
                                                                               .tr_burst_r2(oOOI),
                                                                               .tr_burst_r3(OlOI),
                                                                               .tr_burst_r4(l0OI),
                                                                               .tr_daddr_w1(ioOI),
                                                                               .tr_daddr_w2(lOII),
                                                                               .tr_saddr_r1(oii),
                                                                               .tr_saddr_r2(iOOI),
                                                                               .tr_saddr_r3(IlOI),
                                                                               .tr_saddr_r4(o0OI),
                                                                               .tr_type_w1(IiOI),
                                                                               .tr_type_w2(iOII),
                                                                               .tr_type_r1(OOOI),
                                                                               .tr_type_r2(IIOI),
                                                                               .tr_type_r3(olOI),
                                                                               .tr_type_r4(O1OI),
                                                                               .wdata_din1(oiOI),
                                                                               .wdata_din2(IIII),
                                                                               .rdata_dout(l1o),
                                                                               .write_ack1(o0o),
                                                                               .write_ack2(i0o),
                                                                               .read_ack1(Oi1),
                                                                               .read_ack2(Ii1),
                                                                               .read_ack3(li1),
                                                                               .read_ack4(oi1),
                                                                               .tr_done_w1(IIo),
                                                                               .tr_done_w2(lIo),
                                                                               .tr_done_r1(lOo),
                                                                               .tr_done_r2(oOo),
                                                                               .tr_done_r3(iOo),
                                                                               .tr_done_r4(OIo),
                                                                               .buff_re1(O11),
                                                                               .buff_re2(I11),
                                                                               .buff_we1(l11),
                                                                               .buff_we2(o11),
                                                                               .buff_we3(i11),
                                                                               .buff_we4(Oo1),
                                                                               .tr_trg_w(ilo),
                                                                               .tr_trg_r(olo),
                                                                               .rdata_dout1(Io1),
                                                                               .rdata_dout2(lo1),
                                                                               .rdata_dout3(oo1),
                                                                               .rdata_dout4(io1),
                                                                               .tr_length_w(iIo),
                                                                               .tr_length_r(oIo),
                                                                               .tr_size_w(llo),
                                                                               .tr_size_r(Ilo),
                                                                               .tr_burst_w(OOo),
                                                                               .tr_burst_r(ii1),
                                                                               .tr_daddr_w(IOo),
                                                                               .tr_saddr_r(Olo),
                                                                               .tr_type_w(I0o),
                                                                               .tr_type_r(O0o),
                                                                               .wdata_din(l0o));
  AXI_M #(.g_AXI_AWIDTH(g_AXI_AWIDTH), .g_AXI_DWIDTH(g_AXI_DWIDTH)) OolI(.clock(SYS_CLOCK_I),
                                                                         .nreset(RESET_N_I),
                                                                         .m_awready(m_awready_i),
                                                                         .m_wready(m_wready_i),
                                                                         .m_bvalid(m_bvalid_i),
                                                                         .m_arready(m_arready_i),
                                                                         .m_rlast(m_rlast_i),
                                                                         .m_rvalid(m_rvalid_i),
                                                                         .tr_trg_w(ilo),
                                                                         .tr_trg_r(olo),
                                                                         .m_bid(m_bid_i),
                                                                         .m_bresp(m_bresp_i),
                                                                         .m_rid(m_rid_i),
                                                                         .m_rdata(m_rdata_i),
                                                                         .m_rresp(m_rresp_i),
                                                                         .tr_length_w(iIo),
                                                                         .tr_size_w(llo),
                                                                         .tr_burst_w(OOo),
                                                                         .tr_daddr_w(IOo),
                                                                         .tr_type_w(I0o),
                                                                         .tr_length_r(oIo),
                                                                         .tr_size_r(Ilo),
                                                                         .tr_burst_r(ii1),
                                                                         .tr_saddr_r(Olo),
                                                                         .tr_type_r(O0o),
                                                                         .wdata_din(l0o),
                                                                         .m_awvalid(lIi),
                                                                         .m_wlast(lli),
                                                                         .m_wvalid(ili),
                                                                         .m_bready(oIi),
                                                                         .m_arvalid(oio),
                                                                         .m_rready(iIi),
                                                                         .tr_done_w(i1o),
                                                                         .tr_done_r(o1o),
                                                                         .buff_we(I1o),
                                                                         .buff_re(O1o),
                                                                         .m_awid(lOi),
                                                                         .m_awaddr(iio),
                                                                         .m_awlen(oOi),
                                                                         .m_awsize(IIi),
                                                                         .m_awburst(OOi),
                                                                         .m_awlock(iOi),
                                                                         .m_awcache(IOi),
                                                                         .m_awprot(OIi),
                                                                         .m_wdata(Oli),
                                                                         .m_wstrb(oli),
                                                                         .m_wid(Ili),
                                                                         .m_arid(ooo),
                                                                         .m_araddr(Ooo),
                                                                         .m_arlen(ioo),
                                                                         .m_arsize(lio),
                                                                         .m_arburst(Ioo),
                                                                         .m_arlock(Oio),
                                                                         .m_arcache(loo),
                                                                         .m_arprot(Iio),
                                                                         .rdata_dout(l1o));
  rd_master_req_latch IolI(.clock(SYS_CLOCK_I),
                           .nreset(RESET_N_I),
                           .rd_req_1(rd_req_1_i),
                           .rd_done_1(i0i),
                           .rd_req_2(rd_req_2_i),
                           .rd_done_2(O1i),
                           .rd_req_3(rd_req_3_i),
                           .rd_done_3(I1i),
                           .rd_req_4(rd_req_4_i),
                           .rd_done_4(l1i),
                           .rd_req_arb_1(o1i),
                           .rd_req_arb_2(i1i),
                           .rd_req_arb_3(Ooi),
                           .rd_req_arb_4(Ioi));
  wr_master_req_latch lolI(.clock(SYS_CLOCK_I),
                           .nreset(RESET_N_I),
                           .wr_req_1(wr_req_1_i),
                           .wr_done_1(OoOI),
                           .wr_req_2(wr_req_2_i),
                           .wr_done_2(IoOI),
                           .wr_req_arb_1(l1lI),
                           .wr_req_arb_2(o1lI));
  read_channel1_top #(.g_AXI_AWIDTH(g_AXI_AWIDTH), .g_AXI_DWIDTH(g_AXI_DWIDTH), .g_AXI_BUFF_AWIDTH(g_RD_CHANNEL1_AXI_BUFF_AWIDTH), .g_HORIZONTAL_RESOLUTION(g_RD_CHANNEL1_HORIZONTAL_RESOLUTION), .g_RD_CHANNEL1_VIDEO_DATA_WIDTH(g_RD_CHANNEL1_VIDEO_DATA_WIDTH), .g_RD_CHANNEL1_BUFFER_LINE_STORAGE(g_RD_CHANNEL1_BUFFER_LINE_STORAGE)) oolI(.clock(SYS_CLOCK_I),
                                                                                                                                                                                                                                                                                                                                               .nreset(RESET_N_I),
                                                                                                                                                                                                                                                                                                                                               .arbitr_ack(Oi1),
                                                                                                                                                                                                                                                                                                                                               .axi_if_wen(l11),
                                                                                                                                                                                                                                                                                                                                               .axi_tr_done(lOo),
                                                                                                                                                                                                                                                                                                                                               .rd_req_1(rd_req_1_i),
                                                                                                                                                                                                                                                                                                                                               .axi_data_in(Io1),
                                                                                                                                                                                                                                                                                                                                               .bytes_to_read_1(bytes_to_read_1_i),
                                                                                                                                                                                                                                                                                                                                               .start_read_addr_1(start_read_addr_1_i),
                                                                                                                                                                                                                                                                                                                                               .axi_start(iii),
                                                                                                                                                                                                                                                                                                                                               .master_rd_req(),
                                                                                                                                                                                                                                                                                                                                               .rd_ack_1(O0i),
                                                                                                                                                                                                                                                                                                                                               .rd_burst_split(),
                                                                                                                                                                                                                                                                                                                                               .rd_done_1(i0i),
                                                                                                                                                                                                                                                                                                                                               .rdata_valid_1(loi),
                                                                                                                                                                                                                                                                                                                                               .axi_burst_len(Iii),
                                                                                                                                                                                                                                                                                                                                               .axi_burst_typ(lii),
                                                                                                                                                                                                                                                                                                                                               .axi_saddr(oii),
                                                                                                                                                                                                                                                                                                                                               .axi_tr_dir(OOOI),
                                                                                                                                                                                                                                                                                                                                               .axi_tr_size(IOOI),
                                                                                                                                                                                                                                                                                                                                               .video_rdata_1(illI));
  read_channel2_top #(.g_AXI_AWIDTH(g_AXI_AWIDTH), .g_AXI_DWIDTH(g_AXI_DWIDTH), .g_AXI_BUFF_AWIDTH(g_RD_CHANNEL2_AXI_BUFF_AWIDTH), .g_HORIZONTAL_RESOLUTION(g_RD_CHANNEL2_HORIZONTAL_RESOLUTION), .g_RD_CHANNEL2_VIDEO_DATA_WIDTH(g_RD_CHANNEL2_VIDEO_DATA_WIDTH), .g_RD_CHANNEL2_BUFFER_LINE_STORAGE(g_RD_CHANNEL2_BUFFER_LINE_STORAGE)) iolI(.clock(SYS_CLOCK_I),
                                                                                                                                                                                                                                                                                                                                               .nreset(RESET_N_I),
                                                                                                                                                                                                                                                                                                                                               .arbitr_ack(Ii1),
                                                                                                                                                                                                                                                                                                                                               .axi_if_wen(o11),
                                                                                                                                                                                                                                                                                                                                               .axi_tr_done(oOo),
                                                                                                                                                                                                                                                                                                                                               .rd_req_2(rd_req_2_i),
                                                                                                                                                                                                                                                                                                                                               .axi_data_in(lo1),
                                                                                                                                                                                                                                                                                                                                               .bytes_to_read_2(bytes_to_read_2_i),
                                                                                                                                                                                                                                                                                                                                               .start_read_addr_2(start_read_addr_2_i),
                                                                                                                                                                                                                                                                                                                                               .axi_start(OIOI),
                                                                                                                                                                                                                                                                                                                                               .master_rd_req(),
                                                                                                                                                                                                                                                                                                                                               .rd_ack_2(I0i),
                                                                                                                                                                                                                                                                                                                                               .rd_burst_split(oIOI),
                                                                                                                                                                                                                                                                                                                                               .rd_done_2(O1i),
                                                                                                                                                                                                                                                                                                                                               .rdata_valid_2(ooi),
                                                                                                                                                                                                                                                                                                                                               .axi_burst_len(lOOI),
                                                                                                                                                                                                                                                                                                                                               .axi_burst_typ(oOOI),
                                                                                                                                                                                                                                                                                                                                               .axi_saddr(iOOI),
                                                                                                                                                                                                                                                                                                                                               .axi_tr_dir(IIOI),
                                                                                                                                                                                                                                                                                                                                               .axi_tr_size(lIOI),
                                                                                                                                                                                                                                                                                                                                               .video_rdata_2(O0lI));
  read_channel3_top #(.g_AXI_AWIDTH(g_AXI_AWIDTH), .g_AXI_DWIDTH(g_AXI_DWIDTH), .g_AXI_BUFF_AWIDTH(g_RD_CHANNEL3_AXI_BUFF_AWIDTH), .g_HORIZONTAL_RESOLUTION(g_RD_CHANNEL3_HORIZONTAL_RESOLUTION), .g_RD_CHANNEL3_VIDEO_DATA_WIDTH(g_RD_CHANNEL3_VIDEO_DATA_WIDTH), .g_RD_CHANNEL3_BUFFER_LINE_STORAGE(g_RD_CHANNEL3_BUFFER_LINE_STORAGE)) OilI(.clock(SYS_CLOCK_I),
                                                                                                                                                                                                                                                                                                                                               .nreset(RESET_N_I),
                                                                                                                                                                                                                                                                                                                                               .arbitr_ack(li1),
                                                                                                                                                                                                                                                                                                                                               .axi_if_wen(i11),
                                                                                                                                                                                                                                                                                                                                               .axi_tr_done(iOo),
                                                                                                                                                                                                                                                                                                                                               .rd_req_3(rd_req_3_i),
                                                                                                                                                                                                                                                                                                                                               .axi_data_in(oo1),
                                                                                                                                                                                                                                                                                                                                               .bytes_to_read_3(bytes_to_read_3_i),
                                                                                                                                                                                                                                                                                                                                               .start_read_addr_3(start_read_addr_3_i),
                                                                                                                                                                                                                                                                                                                                               .axi_start(llOI),
                                                                                                                                                                                                                                                                                                                                               .master_rd_req(),
                                                                                                                                                                                                                                                                                                                                               .rd_ack_3(l0i),
                                                                                                                                                                                                                                                                                                                                               .rd_burst_split(O0OI),
                                                                                                                                                                                                                                                                                                                                               .rd_done_3(I1i),
                                                                                                                                                                                                                                                                                                                                               .rdata_valid_3(ioi),
                                                                                                                                                                                                                                                                                                                                               .axi_burst_len(iIOI),
                                                                                                                                                                                                                                                                                                                                               .axi_burst_typ(OlOI),
                                                                                                                                                                                                                                                                                                                                               .axi_saddr(IlOI),
                                                                                                                                                                                                                                                                                                                                               .axi_tr_dir(olOI),
                                                                                                                                                                                                                                                                                                                                               .axi_tr_size(ilOI),
                                                                                                                                                                                                                                                                                                                                               .video_rdata_3(I0lI));
  read_channel4_top #(.g_AXI_AWIDTH(g_AXI_AWIDTH), .g_AXI_DWIDTH(g_AXI_DWIDTH), .g_AXI_BUFF_AWIDTH(g_RD_CHANNEL4_AXI_BUFF_AWIDTH), .g_HORIZONTAL_RESOLUTION(g_RD_CHANNEL4_HORIZONTAL_RESOLUTION), .g_RD_CHANNEL4_VIDEO_DATA_WIDTH(g_RD_CHANNEL4_VIDEO_DATA_WIDTH), .g_RD_CHANNEL4_BUFFER_LINE_STORAGE(g_RD_CHANNEL4_BUFFER_LINE_STORAGE)) IilI(.clock(SYS_CLOCK_I),
                                                                                                                                                                                                                                                                                                                                               .nreset(RESET_N_I),
                                                                                                                                                                                                                                                                                                                                               .arbitr_ack(oi1),
                                                                                                                                                                                                                                                                                                                                               .axi_if_wen(Oo1),
                                                                                                                                                                                                                                                                                                                                               .axi_tr_done(OIo),
                                                                                                                                                                                                                                                                                                                                               .rd_req_4(rd_req_4_i),
                                                                                                                                                                                                                                                                                                                                               .axi_data_in(io1),
                                                                                                                                                                                                                                                                                                                                               .bytes_to_read_4(bytes_to_read_4_i),
                                                                                                                                                                                                                                                                                                                                               .start_read_addr_4(start_read_addr_4_i),
                                                                                                                                                                                                                                                                                                                                               .axi_start(i0OI),
                                                                                                                                                                                                                                                                                                                                               .master_rd_req(),
                                                                                                                                                                                                                                                                                                                                               .rd_ack_4(o0i),
                                                                                                                                                                                                                                                                                                                                               .rd_burst_split(l1OI),
                                                                                                                                                                                                                                                                                                                                               .rd_done_4(l1i),
                                                                                                                                                                                                                                                                                                                                               .rdata_valid_4(Oii),
                                                                                                                                                                                                                                                                                                                                               .axi_burst_len(I0OI),
                                                                                                                                                                                                                                                                                                                                               .axi_burst_typ(l0OI),
                                                                                                                                                                                                                                                                                                                                               .axi_saddr(o0OI),
                                                                                                                                                                                                                                                                                                                                               .axi_tr_dir(O1OI),
                                                                                                                                                                                                                                                                                                                                               .axi_tr_size(I1OI),
                                                                                                                                                                                                                                                                                                                                               .video_rdata_4(l0lI));
  write_channel1_top #(.g_AXI_AWIDTH(g_AXI_AWIDTH), .g_AXI_DWIDTH(g_AXI_DWIDTH), .g_AXI_BUFF_AWIDTH(g_WR_CHANNEL1_AXI_BUFF_AWIDTH), .g_HORIZONTAL_RESOLUTION(g_WR_CHANNEL1_HORIZONTAL_RESOLUTION), .g_WR_CHANNEL1_VIDEO_DATA_WIDTH(g_WR_CHANNEL1_VIDEO_DATA_WIDTH), .g_WR_CHANNEL1_BUFFER_LINE_STORAGE(g_WR_CHANNEL1_BUFFER_LINE_STORAGE)) lilI(.clock(SYS_CLOCK_I),
                                                                                                                                                                                                                                                                                                                                                .rclock(BUFF_READ_CLOCK_I),
                                                                                                                                                                                                                                                                                                                                                .nreset(RESET_N_I),
                                                                                                                                                                                                                                                                                                                                                .arbitr_ack(o0o),
                                                                                                                                                                                                                                                                                                                                                .axi_data_ren(O11),
                                                                                                                                                                                                                                                                                                                                                .axi_tr_done(IIo),
                                                                                                                                                                                                                                                                                                                                                .wdata_valid_1(wdata_valid_1_i),
                                                                                                                                                                                                                                                                                                                                                .wr_req_1(wr_req_1_i),
                                                                                                                                                                                                                                                                                                                                                .bytes_to_write_1(bytes_to_write_1_i),
                                                                                                                                                                                                                                                                                                                                                .start_write_addr_1(start_write_addr_1_i),
                                                                                                                                                                                                                                                                                                                                                .video_wdata_1(video_wdata_1_i),
                                                                                                                                                                                                                                                                                                                                                .axi_start(OiOI),
                                                                                                                                                                                                                                                                                                                                                .master_wr_req(),
                                                                                                                                                                                                                                                                                                                                                .wr_ack_1(o1OI),
                                                                                                                                                                                                                                                                                                                                                .wr_burst_split(iiOI),
                                                                                                                                                                                                                                                                                                                                                .wr_done_1(OoOI),
                                                                                                                                                                                                                                                                                                                                                .axi_burst_len(loOI),
                                                                                                                                                                                                                                                                                                                                                .axi_burst_typ(ooOI),
                                                                                                                                                                                                                                                                                                                                                .axi_daddr(ioOI),
                                                                                                                                                                                                                                                                                                                                                .axi_tr_dir(IiOI),
                                                                                                                                                                                                                                                                                                                                                .axi_tr_size(liOI),
                                                                                                                                                                                                                                                                                                                                                .axi_wr_data_o(oiOI));
  write_channel2_top #(.g_AXI_AWIDTH(g_AXI_AWIDTH), .g_AXI_DWIDTH(g_AXI_DWIDTH), .g_AXI_BUFF_AWIDTH(g_WR_CHANNEL2_AXI_BUFF_AWIDTH), .g_HORIZONTAL_RESOLUTION(g_WR_CHANNEL2_HORIZONTAL_RESOLUTION), .g_WR_CHANNEL2_VIDEO_DATA_WIDTH(g_WR_CHANNEL2_VIDEO_DATA_WIDTH), .g_WR_CHANNEL2_BUFFER_LINE_STORAGE(g_WR_CHANNEL2_BUFFER_LINE_STORAGE)) oilI(.clock(SYS_CLOCK_I),
                                                                                                                                                                                                                                                                                                                                                .rclock(BUFF_READ_CLOCK_I),
                                                                                                                                                                                                                                                                                                                                                .nreset(RESET_N_I),
                                                                                                                                                                                                                                                                                                                                                .arbitr_ack(i0o),
                                                                                                                                                                                                                                                                                                                                                .axi_data_ren(I11),
                                                                                                                                                                                                                                                                                                                                                .axi_tr_done(lIo),
                                                                                                                                                                                                                                                                                                                                                .wdata_valid_2(wdata_valid_2_i),
                                                                                                                                                                                                                                                                                                                                                .wr_req_2(wr_req_2_i),
                                                                                                                                                                                                                                                                                                                                                .bytes_to_write_2(bytes_to_write_2_i),
                                                                                                                                                                                                                                                                                                                                                .start_write_addr_2(start_write_addr_2_i),
                                                                                                                                                                                                                                                                                                                                                .video_wdata_2(video_wdata_2_i),
                                                                                                                                                                                                                                                                                                                                                .axi_start(oOII),
                                                                                                                                                                                                                                                                                                                                                .master_wr_req(),
                                                                                                                                                                                                                                                                                                                                                .wr_ack_2(i1OI),
                                                                                                                                                                                                                                                                                                                                                .wr_burst_split(lIII),
                                                                                                                                                                                                                                                                                                                                                .wr_done_2(IoOI),
                                                                                                                                                                                                                                                                                                                                                .axi_burst_len(OOII),
                                                                                                                                                                                                                                                                                                                                                .axi_burst_typ(IOII),
                                                                                                                                                                                                                                                                                                                                                .axi_daddr(lOII),
                                                                                                                                                                                                                                                                                                                                                .axi_tr_dir(iOII),
                                                                                                                                                                                                                                                                                                                                                .axi_tr_size(OIII),
                                                                                                                                                                                                                                                                                                                                                .axi_wr_data_o(IIII));
endmodule
