/*****************************************************************************************************************************
--
--    File Name    : reset_synchronizer.v 

--    Description  : This module implements two Flop reset synchronization.


-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2018 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
module reset_synchronizer(clock,
                          reset,
                          reset_sync);
  parameter oOlI = 0;
  input clock;
  input reset;
  output reset_sync;
  reg[1 : 0] iOlI;
  generate
    if (oOlI == 1)
      begin
        always
          @(posedge clock or
            posedge reset)
          begin
            if (reset)
              begin
                iOlI[1 : 0] <= 2'b11;
              end
            else
              begin
                iOlI[1 : 0] <= { iOlI[0], 1'b0 };
              end
          end
      end
    else
      begin
        always
          @(posedge clock or
            negedge reset)
          begin
            if (!reset)
              begin
                iOlI[1 : 0] <= 2'b00;
              end
            else
              begin
                iOlI[1 : 0] <= { iOlI[0], 1'b1 };
              end
          end
      end
  endgenerate
  assign reset_sync = iOlI[1];
endmodule
